#include "Driver_EEPROM.h"
#include "DataFlashCommand.h"

#define ARM_FLASH_DRV_VERSION    ARM_DRIVER_VERSION_MAJOR_MINOR(1, 0) /* driver version */



#define FLASH_SECTORS  ARM_FLASH_SECTOR_INFO(0,CONFIG_EEPROM_SIZE_MAX)     //start address, size

/* Sector Information */
#ifdef FLASH_SECTORS
static ARM_FLASH_SECTOR FLASH_SECTOR_INFO[1] = {
    FLASH_SECTORS
};
#else
#define FLASH_SECTOR_INFO    NULL
#endif


#if CONFIG_EEPROM_SIZE_MAX == 512
	#define CONFIG_FLASH_SIZE_MAX			(512UL * DATAFLASH_SECTOR_SIZE)
#elif  CONFIG_EEPROM_SIZE_MAX == 256
	#define CONFIG_FLASH_SIZE_MAX			(256UL * DATAFLASH_SECTOR_SIZE)
#else
	#define CONFIG_FLASH_SIZE_MAX			(CONFIG_EEPROM_SIZE_MAX * DATAFLASH_SECTOR_SIZE)
#endif

const unsigned long DataFlashUserTable[] =
{
	CONFIG_EEPROM_SIZE_MAX,
	CONFIG_RETRY_COUNT,
	CONFIG_FLASH_SIZE_MAX
};

#ifdef __IAR_SYSTEMS_ICC__
#pragma location = "EEPROM1"
//__root const unsigned char __START_flashdata_address[CONFIG_FLASH_SIZE_MAX] ={0xff,0xff};
__root const unsigned char __START_flashdata_address[2] ={0xff,0xff};
#else
const unsigned char __START_flashdata_address[4] __attribute__((section(".eeprom"))) = {0xff,0xff,0xff,0xff};
#endif
  
/* Flash Information */

static ARM_FLASH_INFO FlashInfo = {
    FLASH_SECTOR_INFO,      /* FLASH_SECTOR_INFO  */
    CONFIG_EEPROM_SIZE_MAX, /* FLASH_SECTOR_COUNT */
    0x1,	                /* FLASH_SECTOR_SIZE  */
    1,                      /* FLASH_PAGE_SIZE    */
    1,                      /* FLASH_PROGRAM_UNIT */
    0xff                    /* FLASH_ERASED_VALUE */
};


/* Flash Status */
static ARM_FLASH_STATUS FlashStatus;

/* Driver Version */
static const ARM_DRIVER_VERSION DriverVersion = {
    ARM_FLASH_API_VERSION,
    ARM_FLASH_DRV_VERSION
};

/* Driver Capabilities */
static const ARM_FLASH_CAPABILITIES DriverCapabilities = {
    0, /* event_ready */
    0, /* data_width = 0:8-bit, 1:16-bit, 2:32-bit */
    0  /* erase_chip */
};

//
// Functions
//

ARM_DRIVER_VERSION ARM_Flash_GetVersion(void)
{
  return DriverVersion;
}

ARM_FLASH_CAPABILITIES ARM_Flash_GetCapabilities(void)
{
  return DriverCapabilities;
}

int32_t ARM_Flash_Initialize(ARM_Flash_SignalEvent_t cb_event)
{
  
  return ARM_DRIVER_OK ;
}

int32_t ARM_Flash_Uninitialize(void)
{
  return ARM_DRIVER_OK ;
}

int32_t ARM_Flash_PowerControl(ARM_POWER_STATE state)
{
    switch (state)
    {
    case ARM_POWER_OFF:
        break;

    case ARM_POWER_LOW:
        break;

    case ARM_POWER_FULL:
        break;

    default:
        return ARM_DRIVER_ERROR_UNSUPPORTED;
    }
    
    return ARM_DRIVER_ERROR_UNSUPPORTED ;
}

int32_t ARM_Flash_ReadData(uint32_t addr, unsigned char *data, uint32_t cnt)
{
     int i;
     
     //address check
     if(FlashInfo.sector_info->end < addr)
     {
        return  ARM_DRIVER_ERROR_PARAMETER ;
     }
	 
	 if(cnt > (CONFIG_EEPROM_SIZE_MAX - addr))
	 {
        return  ARM_DRIVER_ERROR_PARAMETER ;	   
	 }

     for(i=0;i<cnt;i++)
     {
        *data++ = DataFlashRead(addr + i);
     }
      return ARM_DRIVER_OK;
}

int32_t ARM_Flash_ProgramData(uint32_t addr, unsigned char *data, uint32_t cnt)
{
	 int i;
     //address check
     if(FlashInfo.sector_info->end < addr)
     {
        return  ARM_DRIVER_ERROR_PARAMETER ;
     }
     if(cnt > (CONFIG_EEPROM_SIZE_MAX - addr))
	 {
        return  ARM_DRIVER_ERROR_PARAMETER ;	   
	 }
	 
     for(i=0;i<cnt;i++)
     {
	     if(DataFlashWrite(addr + i,(unsigned char)*data)!=DATAFLASH_SUCCESS)
	     {
    	   return ARM_DRIVER_ERROR_TIMEOUT ;
	     }
		 (unsigned char*)data++;
     }

	 return ARM_DRIVER_OK ;
}

int32_t ARM_Flash_EraseSector(uint32_t addr)
{
  return ARM_DRIVER_ERROR_UNSUPPORTED ;
}

int32_t ARM_Flash_EraseChip(void)
{
  return ARM_DRIVER_ERROR_UNSUPPORTED ;
}

ARM_FLASH_STATUS ARM_Flash_GetStatus(void)
{
  return FlashStatus;
}

ARM_FLASH_INFO * ARM_Flash_GetInfo(void)
{
  return &FlashInfo;
}

void ARM_Flash_SignalEvent(uint32_t event)
{
}


ARM_DRIVER_FLASH Driver_Eeprom = {
    ARM_Flash_GetVersion,
    ARM_Flash_GetCapabilities,
    ARM_Flash_Initialize,
    ARM_Flash_Uninitialize,
    ARM_Flash_PowerControl,
    ARM_Flash_ReadData,
    ARM_Flash_ProgramData,
    ARM_Flash_EraseSector,
    ARM_Flash_EraseChip,
    ARM_Flash_GetStatus,
    ARM_Flash_GetInfo
};
