/**
  ******************************************************************************
  * @file    board.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the board function prototypes for the C31W73.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef BOARD_H
#define BOARD_H

#ifdef __cplusplus
extern "C" {
#endif

#include <settings.h>
#include "S1C31W73_periph.h"
#include "micron_flash.h"

///< Xtals   ---------------------------------------------------
#define BOARD_CLG_OSC1_XTAL_FREQ    (32768UL)  
#define BOARD_CLG_OSC3_XTAL_FREQ    (12000000UL)
#define BOARD_CLG_EXOSC_FREQUENCY   (0UL)

///< Common  ---------------------------------------------------
#define GET_STR(NAME)              #NAME
#define GET_NAME(NAME)             GET_STR(NAME)

///< CLG TRIMMING ----------------------------------------------
#define BOARD_TRIM_CLG             seCLG_IOSC
#define BOARD_TEMP_CLG             seCLG_OSC3
#define BOARD_TRIM_TERIF           CLG_INTF_IOSCTERIF_Msk
#define BOARD_TRIM_TEDIF           CLG_INTF_IOSCTEDIF_Msk

///< LCD  ------------------------------------------------------
#define BOARD_LCD32C_CLOCK         seLCD32C_IOSC_CLKDIV_512

///< PORT ------------------------------------------------------
#define BOARD_OUTPORT              sePPORT_P01
#define BOARD_INPORT               sePPORT_P02
#define BOARD_PPORT_IRQn           PPORT_IRQn
#define BOARD_PPORT_IRQHandler     PPORT_IRQHandler
#define BOARD_PPORT_CLOCK          sePPORT_OSC1_CLKDIV_1
#define BOARD_USB_PPORT_IRQHandler PPORT_IRQHandler

///< WDT  ------------------------------------------------------
#define BOARD_WDT_CLOCK            seWDT_OSC1_CLKDIV_128

///< UART ------------------------------------------------------
#define BOARD_UART_CH0
//#define BOARD_UART_CH1

#if  defined( UART_PRINTF )        ///< UART_PRINTF for debug
#define BOARD_UART_PRINTF_RX_PORT  sePPORT_P16
#define BOARD_UART_PRINTF_TX_PORT  sePPORT_P17
#define BOARD_UART_PRINTF_CH       UART3_0
#define BOARD_UART_PRINTF_CLK      seUART_OSC3_CLKDIV_1
#endif

///< UART for example software
#ifdef BOARD_UART_CH0
#define BOARD_UART_CLOCK           seUART_OSC3_CLKDIV_1
#define BOARD_UART_RX_PORT         sePPORT_P16
#define BOARD_UART_TX_PORT         sePPORT_P17
#define BOARD_UART_CH              UART3_0
#define BOARD_UART_CLK             seUART_IOSC_CLKDIV_1
#define BOARD_UART_CH_IRQn         UART3_0_IRQn
#define BOARD_UART_CH_IRQHandler   UART3_0_IRQHandler
#endif

#ifdef BOARD_UART_CH1
#define BOARD_UART_CLOCK           seUART_OSC3_CLKDIV_1
#define BOARD_UART_RX_PORT         sePPORT_P27
#define BOARD_UART_TX_PORT         sePPORT_P26
#define BOARD_UART_CH              UART3_1
#define BOARD_UART_CLK             seUART_IOSC_CLKDIV_1
#define BOARD_UART_CH_IRQn         UART3_1_IRQn
#define BOARD_UART_CH_IRQHandler   UART3_1_IRQHandler
#endif

///< SVD  ------------------------------------------------------
#define BOARD_SVD_CLOCK            seSVD_OSC1_CLKDIV_1
#define BOARD_SVD_CH               SVD4
#define BOARD_SVD_IRQn             SVD4_IRQn
#define BOARD_SVD_IRQHandler       SVD4_0_IRQHandler
#define BOARD_EXSVD_PORT           sePPORT_SVD_EXSVD1

///< T16  ------------------------------------------------------
#define BOARD_T16_CH               T16_0
#define BOARD_T16_CH_IRQn          T16_0_IRQn
#define BOARD_T16_CH_IRQHandler    T16_0_IRQHandler
#define BOARD_T16_CLOCK            seT16_IOSC_CLKDIV_1

///< T16B ------------------------------------------------------
#define BOARD_T16B_CLOCK           seT16B_IOSC_CLKDIV_32
#define BOARD_T16B_CH              T16B_0
#define BOARD_T16B_TOUTCAP0        sePPORT_P14
#define BOARD_T16B_TOUTCAP1        sePPORT_P15

///< QSPI ------------------------------------------------------
#define BOARD_QSPI_CH                  QSPI_0
#define BOARD_QSPI_MMA_START_ADDR      0x00080000U
#define BOARD_QSPI_HANDSHAKE_PORT      sePPORT_P00
#define BOARD_QSPI_BUS_SPEED_HZ        100000
#define BOARD_QSPI_T16_CLOCK           seT16_IOSC_CLKDIV_1
#define BOARD_QSPI_PPORT_IRQn          PPORT_IRQn
#define BOARD_QSPI_PPORT_IRQHandler    PPORT_IRQHandler

///< SPI  ------------------------------------------------------
#define BOARD_SPI_CH                  SPIA_0
#define BOARD_SPI_SS_PORT             sePPORT_P03
#define BOARD_SPI_SDI_PORT            sePPORT_P00
#define BOARD_SPI_SDO_PORT            sePPORT_P01
#define BOARD_SPI_CLK_PORT            sePPORT_P02
#define BOARD_SPI_HANDSHAKE_PORT      sePPORT_P27
#define BOARD_MASTER_CS_PORT          BOARD_SPI_SS_PORT 
#define BOARD_SPI_BUS_SPEED_HZ        100000
#define BOARD_SPI_T16_CLOCK           seT16_IOSC_CLKDIV_1
#define BOARD_SPI_PPORT_IRQn          PPORT_IRQn
#define BOARD_SPI_PPORT_IRQHandler    PPORT_IRQHandler

///< ADC  -------------------------------------------------------
#define BOARD_ADC12A_CH                ADC12A_0
#define BOARD_ADC12A_CLK               seT16_IOSC_CLKDIV_16 
#define BOARD_ADC12A_TEMPERATURE_ADIN  seADC12A_ADIN_TEMPERATURE
#define BOARD_ADC12A_TEMPERATURE_INTE  seADC12A_INTE_TEMPERATURE
#define BOARD_ADC12A_START_ADIN        seADC12A_ADIN0
#define BOARD_ADC12A_END_ADIN          seADC12A_ADIN6
#define BOARD_ADC12A_VREFA             sePPORT_P37
#define BOARD_ADC12A_INTE              ( ADC12A_0_INTE_AD0CIE_Msk | \
  ADC12A_0_INTE_AD1CIE_Msk | ADC12A_0_INTE_AD2CIE_Msk | ADC12A_0_INTE_AD3CIE_Msk | \
  ADC12A_0_INTE_AD4CIE_Msk | ADC12A_0_INTE_AD5CIE_Msk | ADC12A_0_INTE_AD6CIE_Msk | \
  ADC12A_0_INTE_AD7CIE_Msk )

///< I2C  ------------------------------------------------------
#define BOARD_I2C_CH0
//#define BOARD_I2C_CH1

#if defined(BOARD_I2C_CH0)         // Channel 0 - P12, P13
#define BOARD_I2C_MASTER_CLOCK     seI2C_IOSC_CLKDIV_1
#define BOARD_I2C_CH               I2C_0
#define BOARD_I2C_SCL_PORT         sePPORT_P12
#define BOARD_I2C_SDA_PORT         sePPORT_P13
#elif defined(BOARD_I2C_CH1)       // Channel 1 - Port pins not initialized
#define BOARD_I2C_MASTER_CLOCK     seI2C_IOSC_CLKDIV_1
#define BOARD_I2C_CH               I2C_1
#define BOARD_I2C_SCL_PORT         sePPORT_None
#define BOARD_I2C_SDA_PORT         sePPORT_None
#endif

//#define BOARD_AK09911C_RST_PORT  sePPORT_P10 

///< REMC ------------------------------------------------------
#define BOARD_REMC_CH                  REMC3
#define BOARD_REMC_REMO_PORT           sePPORT_Designated
#define BOARD_REMC_CLPLS_PORT          sePPORT_None
#define BOARD_REMC_BTN1_PORT           sePPORT_P30
#define BOARD_REMC_BTN2_PORT           sePPORT_P27
#define BOARD_REMC_PPORT_IRQn          PPORT_IRQn
#define BOARD_REMC_PPORT_IRQHandler    PPORT_IRQHandler
#define BOARD_REMC_IOSC_CLK_FREQ       seCLG_IOSC_16MHZ
#define BOARD_REMC_CLOCK               seREMC_IOSC_CLKDIV_4
#define BOARD_REMC_T16_CLOCK           seT16_IOSC_CLKDIV_64
#define BOARD_REMC_T16                 T16_0
#define BOARD_REMC_T16_IRQn            T16_0_IRQn
#define BOARD_REMC_T16_IRQHandler      T16_0_IRQHandler

///< RFC -------------------------------------------------------
#define BOARD_RFC_CLOCK            seRFC_OSC1_CLKDIV_1
#define BOARD_RFC_CH               RFC_0

///< SNDA ------------------------------------------------------
#define BOARD_SNDA_CLOCK           seSNDA_OSC1_CLKDIV_1
#define BOARD_SNDA_CH              SNDA
#define BOARD_BUZZER_POS           sePPORT_Designated
#define BOARD_BUZZER_NEG           sePPORT_None

///< Buttons ---------------------------------------------------
#define BOARD_SW100_PORT           sePPORT_P11
#define BOARD_SW101_PORT           sePPORT_P12
#define BOARD_SW102_PORT           sePPORT_P13
#define BOARD_SW103_PORT           sePPORT_P14
#define BOARD_SW104_PORT           sePPORT_P27

/**
  * @brief  Initializes the Board
  * @retval None
  */
seStatus BoardInit( void );

/**
  * @brief  Receives a byte of data from UART receiver.
  * @note   This function is blocking.
  * @retval one byte of data.
  */
int16_t seUART_getchar( void );

/**
  * @brief  Receives a byte of data from UART receiver.
  * @note   This function is non blocking. It can be called from Interrupt Serice routine since it
  *         its implementation is deterministic.
  * @retval one byte of data.
  */
int16_t seUART_GetCh( void );

/**
  * @brief  Returns a non-zero integer if a key is in the keyboard buffer.
  *         It will not wait for a key to be pressed.
  * @note   This function is non blocking.
  */
int16_t seUART_kbhit(void);

/**
  * @brief  Transmitter a byte of data from UART transmitter.
  * @note   This function is blocking.
  * @retval one byte of data.
  */
int16_t seUART_putchar( int16_t character );

/**
  * @brief  Displays the frequency of the system clock.
  * @retval None
  */
void ShowRunningClock( void );

#ifdef __cplusplus
}
#endif

#endif /* BOARD_H */

