/**
  ******************************************************************************
  * @file    se_common.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions for the COMMON firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2020. All rights reserved.
  ******************************************************************************
  */

#include <stdio.h>
#include "se_common.h"

/**
  * @brief  System protection
  * @param  protect: On or off value, see seWRITE_PROTECT_OFF
  * @retval None
  */
void seProtectSys( seWriteProtect protect ) {

  SYS->PROT = protect;
}

/**
  * @brief  Clamps the given 32-bit value between a low and high value.
  * @param  value: The given value to be clamped
  * @param  low: Minimum value
  * @param  high: Maximum value
  * @retval clamped value
  */
int32_t seClamp32( int32_t value, int32_t low, int32_t high ) {
  
  int32_t tmp = value;
  
  if ( tmp < low ) {
    tmp = low;
  }
  else if ( tmp > high ) {
    tmp = high;
  }
  else{
  }
  
  return tmp;
}

/**
  * @brief  Clamps the given 16-bit value between a low and high value.
  * @param  value: The given value to be clamped
  * @param  low: Minimum value
  * @param  high: Maximum value
  * @retval clamped value
  */
int16_t seClamp16( int16_t value, int16_t low, int16_t high ) {
  
  int16_t tmp = value;
  
  if ( tmp < low ) {
    tmp = low;
  }
  else if ( tmp > high ) {
    tmp = high;
  }
  else{
  }
  
  return tmp;  
}

/**
  * @brief  Prints filename and code line of the failed statement.  Do not call directly, use the seAssert() macro instead.
  * @param  file: Name of the source file.
  * @param  line: Line number this function is being called on.
  * @retval None
  */
void seAssert(uint8_t* file, uint32_t line) {
  
  //printf( "Failed - %s, line %u\r\n", file, line );
}

