/**
  ******************************************************************************
  * @file    se_dmaspi.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the DMA for SPI firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_DMASPI_H
#define SE_DMASPI_H

#include "se_common.h"

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @addtogroup DMAC
  * @{
  */

/** @defgroup DMASPI_Functions
  * @{
  * @brief The Direct Memory Access functions related to SPI.
  */

/**
  * @brief  Sends a data through the SPIx peripheral by DMA.
  * @param  SPIx: SPI peripheral
  * @param  data: Data to be transmitted.
  * @param  size: Data size in number of words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMASPI_TxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size );

/**
  * @brief  Returns the received data by the SPIx peripheral by DMA.
  * @param  SPIx: SPI peripheral
  * @param  data: The received data.
  * @param  size: Data size in number of words.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMASPI_RxHWords( SPIA_0_Type* SPIx, uint16_t data[], uint32_t size );

/**
  * @}
  */ // DMASPI_Functions

/**
  * @}
  */ // DMAC

/**
  * @}
  */ // sePeriphLibrary

#ifdef __cplusplus
}
#endif
#endif  // SE_DMASPI_H

