/**
  ******************************************************************************
  * @file    se_dmausb.h
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file contains all the functions prototypes for the DMA for USB firmware
  *          library.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

#ifndef SE_DMAUSB_H
#define SE_DMAUSB_H

#include <se_common.h>
#include <stdio.h>

#ifdef __cplusplus
 extern "C" {
#endif

/** @addtogroup sePeriphLibrary
  * @{
  */

/** @addtogroup DMAC
  * @{
  */

/** @defgroup DMAUSB_Constants
  * @{
  * @brief  DMA USB Data definition
  */

#define seUSB_DMATRAN_THRESHOLD 64     ///< Setup the threshold for copying by DMAC.

/** @defgroup DMAUSB_Functions
  * @{
  * @brief DMA USB related functions
  */

/**
  * @brief  Initializes DMA and prepares it for USB use.
  * @param  dma_data_struc_ptr: Address of an alocated buffer for use for DMAC descriptors. Must be aligned.
  * @param  DmaChIn: Input DMA channel.
  * @param  DmaChOut: Output DMA channel.
  * @retval Status: can be a value of @ref seStatus
  */
seStatus seDMAUSB_Init( uint32_t dma_data_struc_ptr, seDMAC_CHANNEL DmaChIn, seDMAC_CHANNEL DmaChOut );

/**
  * @brief  Reads available data from FIFO by DMA.
  * @param  EPNum: Endpoint number.
  * @param  buf: Pointer to a destination byte array.
  * @param  size: Destination array size.
  * @retval Returns a number of bytes read.
  */
uint16_t seDMAUSB_ReadFifo( uint16_t EPNum, uint8_t* buf, uint16_t size );

/**
  * @brief  Writes data to available space in FIFO by DMA.
  * @param  EPNum: Endpoint number.
  * @param  buf: Pointer to a source byte array.
  * @param  size: Source array size.
  * @retval Returns a number of bytes written.
  */
uint16_t seDMAUSB_WriteFifo( uint16_t EPNum, const uint8_t* buf, uint16_t size );

/**
  * @}
  */ // DMAUSB_Functions

/**
  * @}
  */ // DMAC

/**
  * @}
  */ // sePeriphLibrary

#ifdef __cplusplus
}
#endif
#endif // SE_DMAUSB_H

