/****************************************************************************
 *
 * @file	main.c
 * @brief	This is a loadable program for S1C31W73 sample boot loader.
 *
 * @par		Copyright:
 *			Copyright (C) SEIKO EPSON CORPORATION 2019. All rights reserved.
 *
 * @warning -
 *
 * @par		History:
 *			- 06/12/2019    1.00
 *				- First revision.
 *
 ***************************************************************************/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "se_pport.h"

/* --- typedef --- */
/* --- extern --- */


static void start_upgrade( uint32_t app_link_location ){
  void (*pfMainApp)( void );
  uint32_t *pointer;

  __disable_irq();

  pointer = (uint32_t *)app_link_location;

  pfMainApp = (void (*)(void))(*pointer);
  pfMainApp();
}

/* --- prototype --- */
int main(void);

/****************************************************************************
 * main function.
 *
 *  1st. Run the function of __vector_table[47].
 *  2nd. By hardware reset, the boot loader will start update process.
 *
 *	return EXIT_SUCCESS:Success.
 *
 ****************************************************************************/
int main(void)
{

    /// call "start_upgrade" function of the boot loader
    start_upgrade(0x00bc);


	/// dummy
	
	/// Set output port
    sePPORT_InitAsOutput( sePPORT_P01 );
    /// Disable interrupt.
    sePPORT_DisableInt( sePPORT_P01 );

	
    while(1){
	    /// Set output data(LOW).
    	sePPORT_SetOutput( sePPORT_P01, sePPORT_DATA_LOW );
		/// Set output data(HIGH).
    	sePPORT_SetOutput( sePPORT_P01, sePPORT_DATA_HIGH );
	
	};

	return EXIT_SUCCESS;
}