/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V1.1
  * @date    17-November-2017
  * @brief   This file is FLASH Driver example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2019. All rights reserved.
  *****************************************************************************
  */ 

/** @defgroup FLASH
  * @{
  * @brief FLASH module

  * \section intro_sec Introduction
  * This example erases and programs the internal flash memory in
  * the S1C31xxx using the seFlashLibrary.
  * The seFlashLibrary is implemented based on the specification 
  * of the CMSIS-Driver Flash Interface.

  * \section operation_sec Operations
  * 
  * -Call the GetInfo() function to get the information on the internal flash memory.
  * -Call the GetVersion() function to get the version of the flash driver.
  * -Call the Initialize() function to initialize the flash driver.
  * -Call the EraseSector() function to erase the sector specified by the argument.
  * -Call the ProgramData() function to write the data to the sector specified in step 4.
  * -Call the ReadData() function to read the data from the sector specified in step 4, and compare the read data with the data written in step 5.
  * -Call the Uninitialize() function to make the flash driver uninitialized.
  *
  * \subsection output Example of output:
  *
  * Erase: OK
  * Program: OK
  * Verify: OK
  * Exit
  *
  ***************************************************************************  
  */
/**
  * @}
  */
/* include */
#include <stdio.h>
#include <string.h>
#include "Driver_Flash.h"

extern  ARM_DRIVER_FLASH Driver_Flash;

unsigned char updateLineBit[16] = {
		0x0F,0x0E,0x0D,0x0C,0x0B,0x0A,0x09,0x08,0x07,0x06,0x05,0x04,0x03,0x02,0x01,0x00
};


int main(void)
{
  unsigned char compbuf[16];
  
  //disable Interrupt
  __asm("CPSID i");
    
    
  ARM_FLASH_INFO *Info = Driver_Flash.GetInfo();
  
  
  Driver_Flash.GetVersion();
  
  
  //Initialize
  Driver_Flash.Initialize(NULL);
  

  //Erase at 0x1D000
  if (Driver_Flash.EraseSector(0x1D000)==ARM_DRIVER_OK)
  {
    printf("Erase: OK\n");
    //Write at 0x1D000
    if (Driver_Flash.ProgramData(0x1D000, updateLineBit,16)==ARM_DRIVER_OK)
    {
      printf("Program: OK\n");
      //Read at 0x1D000
      Driver_Flash.ReadData(0x1D000, compbuf, 16);
      if (memcmp(updateLineBit, compbuf, 16) == 0) {
        printf("Verify: OK\n");
      } else {
        printf("Verify: NG\n");
      }
    } else {
      printf("Program: NG\n");
    }
  } else {
    printf("Erase NG\n");
  }
      
  printf("Exit\n");

  //Uninitialize
  Driver_Flash.Uninitialize();
  
  //enable Interrupt
  __asm("CPSIE i");

	return 0;
}
