/******************************************************************
 *		main.c
 *
 *		This is the main function of the sample test.
 *			call test program.
 *
 * 		history	:	2009.06.04	new making
 *
 ******************************************************************/

#include <stdio.h>
#include "s1c31Selftest.h"
#include "s1c31.h"

extern short s1c31TestRam16(volatile unsigned short *chkAddr, unsigned short chkNum);
extern short s1c31TestRam8(volatile unsigned char *chkAddr, unsigned short chkNum);
extern short s1c31TestRegister(void);
extern short s1c31TestPsr(void);
extern short s1c31TestRamMarchc(volatile unsigned char *startAddr, unsigned short chkNum);
extern unsigned short s1c31TestChksum(unsigned char *startAddr, unsigned short chkNum);
extern unsigned short s1c31TestCrc(unsigned char *startAddr, unsigned short chkNum);
extern unsigned short s1c31TestCrcTbl(unsigned char *startAddr, unsigned short chkNum);
extern unsigned short s1c31TestInterrupt(unsigned short numInt);
extern short s1c31TestClk(unsigned long baseFreq, unsigned short range);


#define TEST_RAM_SA 0x20000000
#define TEST_RAM_SZ 0x0010
#define TEST_DATA_SIZE 10

const unsigned char testData[TEST_DATA_SIZE] = {
		0x0a, 0x25, 0x33, 0xf0, 0x67,
		0x88, 0x1c, 0x4d, 0x76, 0x53
};

/**
 *	main
 *
 *	@brief 	main function of the sample program. call test program
 *
 *	@return int 0
 *
 */
int main(void) {
	short check;
	unsigned short checksum;
	unsigned short crc;
	unsigned short countInt;// = add(2);


//asm("MRS Rd, PSR"); 
	printf("RAM R/W test(16bit) start\n");
	/* execute RAM memory R/W test program for 16bit device. */
	__asm("CPSID I"); // __disable_irq(); /* CPSID I */ 
	check = s1c31TestRam16((unsigned short *)TEST_RAM_SA, TEST_RAM_SZ);
	__asm("CPSIE I"); // __enable_irq(); /* CPSIE I */
  __enable_irq();
	if (check != E_OK) {
		printf("RAM memory error occurred!!!\n\n");
	} else {
		printf("RAM memory is normal.\n\n");
	}



	printf("RAM R/W test(8bit) start\n");
	/* execute RAM memory R/W test program for 8bit device. */
	__asm("CPSID I");
	check = s1c31TestRam8((unsigned char *)TEST_RAM_SA, TEST_RAM_SZ);
	__asm("CPSIE I");
	if (check != E_OK) {
		printf("RAM memory error occurred!!!\n\n");
	} else {
		printf("RAM memory is normal.\n\n");
	}


	printf("register R/W test start:\n");
	/* execute register R/W test program. */
	__asm("CPSID I");
	check = s1c31TestRegister();
	__asm("CPSIE I");
	if (check != E_OK) {
		printf("register error occurred!!!\n\n");
	} else {
		printf("register is normal.\n\n");
	}

	printf("PSR R/W test start:\n");
	/* execute PSR R/W test program. */
	__asm("CPSID I");
	check = s1c31TestPsr();
	__asm("CPSIE I");
	if (check != E_OK) {
		printf("PSR error occurred!!!\n\n");
	} else {
		printf("PSR is normal.\n\n");
	}


	printf("March-c test start:\n");
	/* execute March-c test program. */
	__asm("CPSID I");
	check = s1c31TestRamMarchc((unsigned char *)TEST_RAM_SA, TEST_RAM_SZ);
	__asm("CPSIE I");
	if (check != E_OK) {
		printf("RAM memory error occurred!!!\n\n");
	} else {
		printf("RAM memory is normal.\n\n");
	}

	printf("checksum test start:\n");
	/* execute checksum test program. */
	checksum = s1c31TestChksum((unsigned char*)testData, TEST_DATA_SIZE);
	printf("checksum value = 0x%04x\n\n",checksum);

	printf("crc test start(calculate)\n");
	/* execute crc(calculate) test program. */
	crc = s1c31TestCrc((unsigned char*)testData, TEST_DATA_SIZE);
	printf("CRC value(calculate) = 0x%04x\n\n",crc);

	printf("crc test start(use table)\n");
	/* execute crc(table) test program. */
	crc = s1c31TestCrcTbl((unsigned char*)testData, TEST_DATA_SIZE);
	printf("CRC value(use table) = 0x%04x\n\n",crc);

	/* execute interrupt count test program. */
	printf("interrupt count test start (push SW14):\n");
	countInt = s1c31TestInterrupt(0x3);
	printf("port interrupt occurred %d.\n\n",countInt);

	/* execute main clock test program. */
	printf("main clock test start:\n");
	if ((s1c31TestClk(seCLG_IOSC_24MHZ/*33333333*/, 1)) != E_OK) {
		printf("main clock error occurred!!!\n");
	} else {
		printf("main clock is normal.\n");
	}


	return 0;
}
