/******************************************************************
 *		s1c31TestInterrupt.c
 *
 *		count interrupt(sw7) number while the constant period.
 *
 * 		history	:	2009.06.04	new making
 *
 ******************************************************************/

#ifdef PRINT_SIMIO
#include <stdio.h>
#endif
//#include "s1c31.h"
#include "se_common.h"

/* prototype */
unsigned short s1c31TestInterrupt(unsigned short numInt);
static void s1c31InitIntTestInt(void);

/* define 16-bit timer and port1 interrupt flag status */
#define T16F_INT_ON 1
#define T16F_INT_OFF 0
#define T16F_INT_LEVEL 5
#define T16F_COUNT_DATA 0xf000
#define PORT1_INT_ON 1
#define PORT1_INT_OFF 0
#define PORT1_INT_LEVEL 3


volatile unsigned short t16fIntFlag;		// 16-bit timer interrupt flag
volatile unsigned short port1IntFlag;		// port1 interrupt flag
/**
 *	s1c31TestInterrupt
 *
 *	@brief 	count port interrupt till timer interrupt occur numInt times.
 *
 *	@param1 numInt		number of timer interrupt.
 *
 *	@return unsigned short
 *			times of port1 interrupt occurred
 */
unsigned short s1c31TestInterrupt(unsigned short numInt)
{
#if 1
	unsigned short loop = 0;				//loop variable
	volatile unsigned short countInt = 0;	//port interrupt counter
        T16_0_Type* T16x = T16_4;

	t16fIntFlag = T16F_INT_OFF;
	port1IntFlag = PORT1_INT_OFF;

	/* initialize 16-bit timer and port1 interrupt */
	s1c31InitIntTestInt();

#ifdef PRINT_SIMIO
	printf("count port1 interrupt start!!\n");
#endif

	/* disable interrupt */
	__asm("CPSID I");

	/* 16-bit timer start */
	T16x->CTL_b.PRUN = 1; //? T16F_CTL_PRUN = 0x01;

	/* port1 interrupt enable */
	T16x->INTF = 0x01;   //? T16F_INT_T16IF = 0x1;
//	P1_IMSK_P1IE |= 0x08;
    PPORT->P1INTCTL_b.P1IE0 = 1;
//	PPORT->P1INTCTL_b.P1RIE0 = 1;
  
  
	while (loop < numInt) {
		__asm("CPSIE I");
		__WFI();//__asm("halt");
		__asm("nop");
		__asm("CPSID I");
		__asm("nop");
		__asm("nop");

		/* 16-bit timer interrupt occurred */
		if (t16fIntFlag == T16F_INT_ON) {
			t16fIntFlag = T16F_INT_OFF;
			loop++;
		} else {
			/* nothing to do */
		}

		/* port1 interrupt occurred */
		if (port1IntFlag == PORT1_INT_ON) {
			countInt++;
			port1IntFlag = PORT1_INT_OFF;
		} else {
			/* nothing to do */
		}
	}

	/* enable interrupt */
	__asm("CPSIE I");


	/* stop port interrupt */
	//P1_IMSK_P1IE &= (~0xff);
    PPORT->P1INTCTL_b.P1IE0 = 0;
//  PPORT->P1INTCTL_b.P1RIE0 = 0;

	/* stop timer */
	T16x->CTL_b.PRUN = 0; //? T16F_CTL_PRUN = 0x00;

	/* stop timer interrupt */
	T16x->INTE_b.UFIE = 0; //? T16F_INT_T16IE = 0x0;

	return countInt;
#else
        return 0;
#endif

}



/**
 *	s1c31InitIntTestInt
 *
 *	@brief 	initialize interrupt function 16-bit timer and port1.
 *
 */
static void s1c31InitIntTestInt(void)
{
#if 1
	T16_0_Type* T16x = T16_4;

        /* init 16-bit timer */
	/* stop timer */
	T16x->CTL_b.PRUN = 0; //? T16F_CTL_PRUN = 0x00;

	/* 16-bit timer interrupt flag clear */
	T16x->INTF = 0x01;   //? T16F_INT_T16IF = 0x1;

	/* 16-bit timer interrupt disable */
	T16x->INTE_b.UFIE = 0; //? T16F_INT_T16IE = 0x0;

	/* set interrupt level.5(16-bit timer) */
	//ITC_LV4_ILV8 = T16F_INT_LEVEL;
  NVIC_SetPriority( T16_4_IRQn, T16F_INT_LEVEL ); 
  NVIC_ClearPendingIRQ( T16_4_IRQn ); 
  NVIC_EnableIRQ( T16_4_IRQn );

	/* stop 16-bit timer */
	T16x->CTL_b.PRUN = 0; //? T16F_CTL_PRUN = 0x00;

	/* set count mode repeat */
	T16x->MOD_b.TRMD = 0; //? T16F_CTL_TRMD = 0x00;

	/* set clock 1/1024 */
	//T16F_CLK_DF = 0x0a;
  T16x->CLK_b.CLKSRC = seSRC(seT16B_IOSC_CLKDIV_1024); ///< Clock source selection
  T16x->CLK_b.CLKDIV = seDIV(seT16B_IOSC_CLKDIV_1024); ///< Clock division ratio selection
  T16x->CLK_b.DBRUN  = seDBRUN;      ///< Whether clock supplied in DEBUG mode or not.

  T16x->CTL_b.MODEN = 1;
  
	/* set reload data */
	T16x->TR = T16F_COUNT_DATA;   //? T16F_TR_TR = T16F_COUNT_DATA;

	/* timer reset */
	T16x->CTL_b.PRESET = 1; //? T16F_CTL_PRESER = 0x01;

	/* fine mode off*/
//	T16F_CTL_TFMD = 0x0;

	/* init port */
	/* disable port function protection */
#if 0  
	PF_WREN_WREN = 0x96;

	P4_PMUX0_3_P42MUX = 0x0;	// P42(R-ENC_A)
	P4_PMUX0_3_P43MUX = 0x0;	// P43(R-ENC_B)

	/* enable port function protection */
	PF_WREN_WREN = 0x00;
#endif

  /* set P15 as output */
  PPORT->P1MODSEL_b.P1SEL5 = 0; // Enable GPIO function.
  PPORT->P1IOEN_b.P1IEN5 = 0;   // Disable input port.
  PPORT->P1IOEN_b.P1OEN5 = 1;   // Enable output port.
  /* set P15 output */
  PPORT->P1DAT_b.P1OUT5 = 0;    // Set to low
  
  /* set P10 as input */
  PPORT->P1RCTL_b.P1PDPU0 = 0;
  PPORT->P1RCTL_b.P1REN0 = 0;
  PPORT->P1MODSEL_b.P1SEL0 = 0; // Enable GPIO function.
  PPORT->P1IOEN_b.P1OEN0 = 0;   // Disable output port.
  PPORT->P1IOEN_b.P1IEN0 = 1;   // Enable input port.
  PPORT->P1RCTL_b.P1PDPU0 = 1;
  PPORT->P1RCTL_b.P1REN0 = 1;
  

  
	/* interrupt stop */
	//P1_IMSK_P1IE &= (~0xff);
  PPORT->P1INTCTL = 0;

	/* set interrupt level.3 */
	//ITC_LV1_ILV2 = PORT1_INT_LEVEL;
  NVIC_SetPriority( PPORT_IRQn, 2 ); 
  NVIC_ClearPendingIRQ( PPORT_IRQn ); 
  NVIC_EnableIRQ( PPORT_IRQn );  

	/* port interrupt 1 selector */
	//PP_SEL1_PSEL1 = 0x0c;

	/* port interrupt 1 edge -> edge trigger mode*/
	//P1_EDGE_P1EDGE = 0x00;
  //PPORT->P1INTCTL_b.P1EDGE0 = 0;

	/* port interrupt 1 polarity control */
	//P1_POL_P1POL = 0x0c;

	/* Reset port1 interrupt flag */
	//P1_IFLG_P1IF = 0x0c;

	/* 16bit-timer interrupt flag clear */
	T16x->INTF = 0x01;   //? T16F_INT_T16IF = 0x1;

	/* 16-bit timer interrupt enable */
	T16x->INTE_b.UFIE = 1; //? T16F_INT_T16IE = 0x1;

	/* port1 interrupt flag clear */
	//P1_IFLG_P1IF = 0xff;
  PPORT->P1INTF = 0xff;
#endif
}

/**
 *	T16fInt
 *
 *	@brief 	16bit timer interrupt handler.
 */
void T16_4_IRQHandler(void)
{
       // T16_0_Type* T16x = T16_4;

	/* 16bit-timer interrupt flag clear */
	T16_4->INTF = 0x01;   //? T16F_INT_T16IF = 0x1;
	t16fIntFlag = T16F_INT_ON;
}

/**
 *	P1Int
 *
 *	@brief 	port1 interrupt handler.
 */
//void s1c31w73P1Int(void)
void PPORT_IRQHandler(void)
{
#if 1
//  if( PPORT->P1INTF & PPORT_P1INTF_P1RIF0_Msk ) {
//		PPORT->P1INTF = PPORT_P1INTF_P1RIF0_Msk;
  if( PPORT->P1INTF & PPORT_P1INTF_P1IF0_Msk ) {
		PPORT->P1INTF = PPORT_P1INTF_P1IF0_Msk;
	}
	port1IntFlag = PORT1_INT_ON;
#endif
}

