/******************************************************************
 *		s1c31TestPsr.c
 *
 *		This program execute the PSR R/W test.
 *
 * 		history	:	2009.06.03	new making
 *
 ******************************************************************/
//#include "cmsis_iar.h"
//#include "core_cmFunc.h"
#include "s1c31SelfTest.h"

/* prototype */
extern void s1c31WritePsr(void);
extern void s1c31ReadPsr(void);
short s1c31TestPsr(void);

/* 
 * The Program Status Register (PSR) combines:
 * 1. Application Program Status Register (APSR). It is RW for 31-28bits and 27-0bits reserved.
 * 2. Interrupt Program Status Register (IPSR). It is RO and such we do not care.
 * 3. Execution Program Status Register (EPSR). It is RO and such we do not care.
 */
#define TEST1_VALUE 0x50000000
#define TEST2_VALUE 0xa0000000

/* Global variable define */
unsigned int wPsrVal;				// the value write to PSR
unsigned int backupPsrVal;			// the variable to backup PSR
//volatile short result;				// result of PSR R/W test

//unsigned int __asm processor_in_user_mode(void)
//{
//        STMDB   R13!, {R1}

//        MRS     R0, CPSR
//        AND     R0, R0, #0x1F   // the lowest 5 bits of CPSR respresent the processor mode
//        MOV             R1, #0x10               // user mode = 0x10
//        CMP             R0, R1
//        BEQ             not_interrupt_context

//        LDMIA   R13!, {R1} // restore the file name and the line number.

//        MOV             R0,     #0x0 // indicate called while not in user mode

//        BX              LR

//not_interrupt_context

//        // restore original values of R1
//        LDMIA   R13!, {R1}

//        MOV             R0,     #0x1 // indicate called while in user mode

//        BX              LR
//}

//#pragma ARM  // would do the same as __arm below !
//void ChangeIRQ (unsigned int NewState) { __arm  {  // use ARM-mode for this function
//  __asm  {
//     AND    R0,R0,#0         //
//     MRS    R0,CPSR          //
//     ORR    R0,R0,#0x80;     //

//     LDAV   R1,R10,NewState  // load parameter-value 'NewState' into R1
//     BIC    R0,R0,R1,LSL #7  //
//     MSR    CPSR_c, R0       //
//  }
//}

//int qadd(int i, int j)
//{
//    int res;
//    __asm
//    {
//        ADD   res, i, j
//    }
//    return res;
//}

/**
 *	s1c31TestPsr
 *
 *	@brief 	execute the PSR R/W test.
 *
 *	@return E_OK or E_REGISTER
 *			pass the test -> E_OK
 *			failed the test -> E_MEMORY
 */

short s1c31TestPsr(void)
{ 
#ifdef __IAR_SYSTEMS_ICC__

#pragma location=apsr
  int apsr;
#pragma location=R1  
  int R1;	
#else	
  register int apsr  __asm("apsr");
  register int R1    __asm("r1");
#endif
	
  backupPsrVal = apsr;
  
  wPsrVal = TEST1_VALUE;
  apsr = wPsrVal;
  R1 = apsr;
  
  if ( R1 != wPsrVal ) 
    goto ERR;
  
  wPsrVal = TEST2_VALUE;
  apsr = wPsrVal;
  
  if ( apsr != wPsrVal ) 
    goto ERR;  
  
  apsr = backupPsrVal;
  
  return E_OK;
ERR:
  return E_REGISTER;
  
  
#if 0
	/* cause interrupt write to PSR the value wPsrVal. */
	asm("int	30");

	/* cause interrupt read PSR and compare the value wPsrVal. */
	asm("int	31");

	if (result != E_OK) {
		return result;
	} else {
		/* nothing to do */
	}

	wPsrVal = TEST2_VALUE;

	/* cause interrupt write to PSR the value wPsrVal. */
	asm("int	30");

	/* cause interrupt read PSR and compare the value wPsrVal. */
	asm("int	31");

	if (result != E_OK) {
		return result;
	} else {
		/* nothing to do */
	}
#endif
//	return result;
}


