/******************************************************************
 *		s1c31TestRam16.c
 *
 *		This program execute the RAM R/W test in 16 bits.
 *
 * 		history	:	2009.05.29	new making
 *
 ******************************************************************/

#include "s1c31SelfTest.h"

/* prototype */
short s1c31TestRam16(volatile unsigned short *chkAddr, unsigned short chkNum);

#define MASK_ADDRESS 0xfffffffe
#define TEST1_VALUE 0x55aa
#define TEST2_VALUE 0xaa55

 /**
  *	s1c31TestRam16
  *
  *	@brief 	execute the RAM R/W test for 16bit device.
  *
  *	@param1	chkAddr		start Address for test.
  *	@param2 chkNum		number of check memories.
  *
  *	@return E_OK or E_MEMORY
  *			pass the test -> E_OK
  *			failed the test -> E_MEMORY
  */
short s1c31TestRam16(volatile unsigned short *chkAddr, unsigned short chkNum)
{
	unsigned short backup;			//backup for RAM
	unsigned short loop;			//loop variable
	// chkAddr should not odd number.
	chkAddr = (unsigned short*)((unsigned long)chkAddr & MASK_ADDRESS);


	for (loop = 0; loop < chkNum; loop++) {
		backup = *chkAddr;
		*chkAddr = TEST1_VALUE;
		if (*chkAddr != TEST1_VALUE) {
			/* RAM memory error occurred. */
			*chkAddr = backup;
			return E_MEMORY;
		} else {
			/* nothing to do */
		}

		*chkAddr = TEST2_VALUE;

		if (*chkAddr != TEST2_VALUE) {
			/* RAM memory error occurred. */
			*chkAddr = backup;
			return E_MEMORY;
		} else {
			/* nothing to do */
		}

		*chkAddr = backup;
		chkAddr++;
	}
	return E_OK;
}
