/******************************************************************
 *		s1c31TestRamMarchc.c
 *
 *		This program execute the RAM March-c test.
 *
 * 		history	:	2009.05.29	new making
 *
 ******************************************************************/

#include "s1c31SelfTest.h"

/* prototype */
short s1c31TestRamMarchc(volatile unsigned char *chkAddr, unsigned short chkNum);
static short s1c31TestBitChkw0(volatile unsigned char *addr);
static short s1c31TestBitChkw1(volatile unsigned char *addr);

/**
 *	s1c31TestRamMarchc
 *
 *	@brief 	execute the RAM March-c test.
 *
 *	@param1	chkAddr		start Address for test.
 *	@param2 chkNum		number of check memories.
 *
 *	@return E_OK or E_MEMORY
 *			pass the test -> E_OK
 *			failed the test -> E_MEMORY
 */
short s1c31TestRamMarchc(volatile unsigned char *chkAddr, unsigned short chkNum)
{
	volatile unsigned char *endAddr;		//end of test RAM
	unsigned short loop;					//loop variable

	endAddr = chkAddr + chkNum - 1;

	/* test memory area is cleared by 0x00 */
	for (loop = 0; loop < chkNum; loop++) {
		*(chkAddr + loop) = 0x00;
	}

	/* W/R check by writing 1 from head */
	for (loop = 0; loop < chkNum; loop++) {
		if(s1c31TestBitChkw1(chkAddr + loop) != E_OK){
			return E_MEMORY;
		} else {
			/* nothing to do */
		}
	}

	/* W/R check by writing 0 from head */
	for (loop = 0; loop < chkNum; loop++) {
		if(s1c31TestBitChkw0(chkAddr + loop) != E_OK){
			return E_MEMORY;
		} else {
			/* nothing to do */
		}
	}

	/* W/R check by writing 1 from end */
	for (loop = 0; loop < chkNum; loop++) {
		if(s1c31TestBitChkw1(endAddr - loop) != E_OK){
			return E_MEMORY;
		} else {
			/* nothing to do */
		}
	}

	/* W/R check by writing 0 from end */
	for (loop = 0; loop < chkNum; loop++) {
		if(s1c31TestBitChkw0(endAddr - loop) != E_OK){
			return E_MEMORY;
		} else {
			/* nothing to do */
		}
	}

	return E_OK;
}

/**
 *	s1c31TestBitChkw0
 *
 *	@brief 	write memory to 0 from the lower bit.
 *
 *	@param1	addr  check Address.
 *
 *	@return E_OK or E_MEMORY
 *			pass the test -> E_OK
 *			failed the test -> E_MEMORY
 */
static short s1c31TestBitChkw0(volatile unsigned char *addr)
{

	if (*addr != 0xff) {			// check all bit is 1
		return E_MEMORY;
	}

	*addr &= 0xfe;					// change 0th bit. 1 -> 0
	if (*addr != 0xfe) {
		return E_MEMORY;
	}

	*addr &= 0xfd;					// change 1st bit. 1 -> 0
	if (*addr != 0xfc) {
		return E_MEMORY;
	}

	*addr &= 0xfb;					// change 2nd bit. 1 -> 0
	if (*addr != 0xf8) {
		return E_MEMORY;
	}

	*addr &= 0xf7;					// change 3rd bit. 1 -> 0
	if (*addr != 0xf0) {
		return E_MEMORY;
	}

	*addr &= 0xef;					// change 4th bit. 1 -> 0
	if (*addr != 0xe0) {
		return E_MEMORY;
	}

	*addr &= 0xdf;					// change 5th bit. 1 -> 0
	if (*addr != 0xc0) {
		return E_MEMORY;
	}

	*addr &= 0xbf;					// change 6th bit. 1 -> 0
	if (*addr != 0x80) {
		return E_MEMORY;
	}

	*addr &= 0x7f;					// change 7th bit. 1 -> 0
	if (*addr != 0x00) {
		return E_MEMORY;
	}

	return E_OK;
}

/**
 *	s1c31TestBitChkw1
 *
 *	@brief 	write memory to 1 from the lower bit.
 *
 *	@param1	addr  check Address.
 *
 *	@return E_OK or E_MEMORY
 *			pass the test -> E_OK
 *			failed the test -> E_MEMORY
 */
static short s1c31TestBitChkw1(volatile unsigned char *addr)
{
	if (*addr != 0x00) {			// check all bit is 0
		return E_MEMORY;
	}

	*addr |= 0x01;					// change 0th bit. 0 -> 1
	if (*addr != 0x01) {
		return E_MEMORY;
	}

	*addr |= 0x02;					// change 1st bit. 0 -> 1
	if (*addr != 0x03) {
		return E_MEMORY;
	}

	*addr |= 0x04;					// change 2nd bit. 0 -> 1
	if (*addr != 0x07) {
		return E_MEMORY;
	}

	*addr |= 0x08;					// change 3rd bit. 0 -> 1
	if (*addr != 0x0f) {
		return E_MEMORY;
	}

	*addr |= 0x10;					// change 4th bit. 0 -> 1
	if (*addr != 0x1f) {
		return E_MEMORY;
	}

	*addr |= 0x20;					// change 5th bit. 0 -> 1
	if (*addr != 0x3f) {
		return E_MEMORY;
	}

	*addr |= 0x40;					// change 6th bit. 0 -> 1
	if (*addr != 0x7f) {
		return E_MEMORY;
	}

	*addr |= 0x80;					// change 7th bit. 0 -> 1
	if (*addr != 0xff) {
		return E_MEMORY;
	}

	return E_OK;
}

