/** @defgroup PPORT
  * @{
  * @brief The PPORT module controls the general-purpose input/output (GPIO) pins of the S1C31W73.
  * 
  * \section intro_sec Introduction
  * This example configures pin P01 as an output connected to pin P02 configured as an input.
  * It performs various tests to check the connectivity of P01 and P02.
  * 
  * 
  * \section hardware_setup Hardware Setup
  *
  * For this example program, connect P01 to P02.
  * 
  *    --------
  *
  *     P01 ------>>------- P02
  *
  *    --------
  *
  *
  * \section operation_sec Operations
  * 
  * 1. Start the OSC1 oscillation and switch the system clock from IOSC to OSC1. Then sotp the IOSC. 
  * 2. Initialize the ports. 
  * 3. Set the P01 port to output and Output a Low level signal. 
  * 4. Set the P02 port to input and set it so that an interrupt occurs when the level changes from Low to High. 
  * 5. Output a High level signal from the P01 port. 
  * 6. Confirm that the P02 port is at High level after an interrupt in the P02 port. 
  * 7. Set the P02 port so that an interrupt occurs when the level changes from High to Low. 
  * 8. Output a Low level signal from the P01 port. 
  * 9. Confirm that the P02 port is at Low level after an interrupt in the P02 port. 
  *
  * \subsection output Example of output:
  * 
  *  -CPU clock- seCLG_IOSC
  * P01 output state: Low
  * P02 input state: Low
  * P02 input waits for RISING edge interrupt
  * P01 output is going to HIGH...
  * Interrupt on P02 occurred
  * P01 output state: High
  * P02 input state: High
  * 
  * P01 output state: High
  * P02 input state: High
  * P02 input waits for FALLING edge interrupt
  * P01 output is going to LOW...
  * Interrupt on P02 occurred
  * P01 output state: Low
  * P02 input state: Low
  * 
  * Exit
  *
  ***************************************************************************  
  */
/**
  * @}
  */ 



