/** @defgroup QSPI_SLAVE
 * @{
 * @brief QSPI module is a Quad Synchronous Serial Interface supporting both master and slave modes. 
 *
 * \section intro_sec Introduction
 *
 * This example provides a description of how to use a QSPI in the slave mode.
 * 
 * \section hardware_setup Hardware Setup
 *
 * 1. The EVT31W73 board has an onboard serial flash device attached to the QSPI interface.
 *    In order to run the QSPI master/slave sample programs, the serial flash on each board must
 *    be isolated from the QSPI interface by removing 0-ohm resistors R157, R158, R159, R160,
 *    R163, and R168.
 *    NOTE:  Ensure that the 0-ohm connections for these resistors are restored after
 *    evaluating the QSPI master/slave sample programs.
 *
 * 2. Connect the evaluation boards where the QSPI master/slave sample programs are installed. 
 *    Connect each port as follows:
 *
 *                     DUAL or QUAD mode
 *         [master]                         [slave]
 *
 *   TH14   QSDI0n3------------><-----------QSDIOn3   TH14
 *
 *   TH13   QSDI0n2------------><-----------QSDIOn2   TH13
 *
 *   TH12   QSDI0n1------------><-----------QSDIOn1   TH12
 *
 *   TH11   QSDI0n0------------><-----------QSDIOn0   TH11
 *
 *   TH10  QSPICLKn------------>>-----------QSPICLKn  TH10
 *
 *   TH15  #QSPISSn------------>>-----------#QSPISSn  TH15
 *
 *   TH1        P00------------>>-----------P00       TH1
 *
 *   TH9        GND------------><-----------GND       TH9
 *
 *                
 *   For definition of THx see EVT31W73 schematic in EVT31W73 manual.              
 *   Please note that the connections are different for single mode.  
 *                      
 *                        SINGLE mode
 *                       
 *         [master]                         [slave]
 *
 *   TH11   QSDI0n0------------->>-----------QSDIOn1  TH12        
 *
 *   TH12   QSDI0n1------------><-----------QSDIOn0   TH11
 *
 *   TH10  QSPICLKn------------>>-----------QSPICLKn  TH10
 *
 *   TH15  #QSPISSn------------>>-----------#QSPISSn  TH15
 *
 *   TH1        P00------------>>-----------P00       TH1
 *
 *   TH9        GND------------><-----------GND       TH9
 *
 * 
 * 3. Launch the slave example program first, then the master program
 * 
 * \section operation_sec Operation
 * 
 * 1. Initialize the QSPI module in slave mode as below:
 *    - Data length is 8bit
 *    - Data format is MSB first
 * 2. Assign GPIO P00 as input and enable interrupt on rising edge to detect a start of communication
 * 3. Receive the 'BUF_SIZE' bytes of random data from the master
 * 4. Send the 'BUF_SIZE' bytes of data back to the master
 *
 ***************************************************************************
 */
/**
  * @}
  */ 


