/** @defgroup SPIA_MASTER
 * @{
 * @brief SPIA module is a synchronous serial interface supporting both master and slave modes. Only one SPIA channel is available in S1C31W65 microcontroller.
 *
 * \section intro_sec Introduction
 *
 * This example provides a description of how to use a SPIA channel in the master mode to transfer data buffer.
 *
 * \section hardware_setup Hardware Setup
 *
 * 1. Connect the evaluation boards where the SPIA master/slave sample programs are installed. Connect each port as follows:
 * 
 *         [master]                         [slave]
 *      TH1    SDIn------------<<-----------SDOn     TH2
 *
 *      TH2    SDOn------------>>-----------SDIn     TH1
 *
 *      TH3 SPICLKn------------>>-----------SPICLKn  TH3
 *
 *      TH4     P03------------>>-----------#SPISSn  TH4
 *
 *      TH7     P27------------>>-----------P27      TH7
 *
 *      TH9     GND------------><-----------GND      TH9
 * 
 * 2. Launch the slave example program first, then the master program
 * 
 * \section operation_sec Operation
 * 
 * 1. Initialize the SPIA module in master mode as below:
 *    - Data length is 8bit
 *    - Data format is MSB first
 *    - Use 16-bit timer T16_1 for baud rate generator.
 * 2. Set bus speed to 1000000.
 * 3. Assign GPIO P27 as output to set Read/Write command to the slave.
 * 4. Send the 'BUF_SIZE' bytes of random data to the slave.
 * 5. Receive the 'BUF_SIZE' bytes of data from the slave.
 * 6. Compare whether the received data is the same as the sent data and exit.
 *
 * \subsection output Example of output:
 *
 *  -CPU clock- seCLG_IOSC
 * Set bus speed 1000000
 * Get bus speed 1000000
 * OK(1), NG(0)
 * OK(2), NG(0)
 * OK(3), NG(0)
 * OK(4), NG(0)
 * OK(5), NG(0)
 * OK(6), NG(0)
 *
 * The SPIA software shows methods of calling the Peripheral Library modules to interface Hardware
 * components on this chip. The sample programs can be built and executed on the Epson SVT board.
 *
 *************************************************************************** 
 */
/**
  * @}
  */ 

