/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V2.0
  * @date    01-June-2020
  * @brief   This file is TSRVR example software.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2018-2020. All rights reserved.
  ******************************************************************************
  */

/** @defgroup TSRVR
  * @{
  * @brief The TSRVR module provides temperature reading using a successive approximation type 12-bit A/D conversion interface.
  * 
  * \section intro_sec Introduction
  * This example shows how to program the TSRVR and ADC12A to take temperature readings.
  * 
  * \section operation_sec Operations
  * 
  * - Start TSRVR Example 
  * - Observe data output in the console window
  *
  * \subsection output Example of output:
  *
  *  -CPU clock- seCLG_IOSC
  * Temperature = ..
  * Temperature = ..
  * Temperature = ..
  * 
  ******************************************************************************
  */
/**
  * @}
  */
#include <string.h>
#include <stdio.h>
#include "board.h"
#include "port_config.h"
#include "tsrvr_temperature.h"

///< Structure for peripheral library

int main( void )  {

  int16_t temperature;

  ///< Displays the frequency of the system clock.
  ShowRunningClock();
  
  ///< Setting of PORT VREFA0
  sePPORT_Init(sePPORT_ADC_0_VREFA, VREFA0_INIT);   ///< Assign port to ADC12 VREFA0

  ///< Set up temperature readings
  initTempTSRVR ( seTSRVR_VREF_2V0, 15 );
  
  seSysSleepMS(10);  ///< wait for printf characters to flush
  
  while( 1 ) {
     ///< Read temperature
     temperature = readTempTSRVR(16);  ///< Read temperature, average 16 ADC samples

     printf("Temperature = %d\n", temperature);
  }
} 

