/**
  ******************************************************************************
  * @file    main.c
  * @author  Epson
  * @version V1.1
  * @date    23-January-2017
  * @brief   This demo(test) demonstrates one of the USB CDC use cases.
  ******************************************************************************
  * @attention
  *
  * Copyright(C) SEIKO EPSON CORPORATION 2016-2019. All rights reserved.
  ******************************************************************************
  */

#include "main.h"

#include <string.h>
#include <stdio.h>
#include "RTL.h"
#include "rl_usb.h"
#include "board.h"

/* Following functions could be used by user if CDC_ACM class is successfully initialized. */
/* USB Device CDC ACM class user functions                                    
int32_t  USBD_CDC_ACM_PortSetLineCoding         (CDC_LINE_CODING *line_coding);
int32_t  USBD_CDC_ACM_PortGetLineCoding         (CDC_LINE_CODING *line_coding);
int32_t  USBD_CDC_ACM_PortSetControlLineState   (uint16_t ctrl_bmp);
int32_t  USBD_CDC_ACM_DataSend                  (const uint8_t *buf, int32_t len);
int32_t  USBD_CDC_ACM_DataFree                  (void);
int32_t  USBD_CDC_ACM_PutChar                   (const uint8_t  ch);
int32_t  USBD_CDC_ACM_DataRead                  (      uint8_t *buf, int32_t len);
int32_t  USBD_CDC_ACM_GetChar                   (void);
int32_t  USBD_CDC_ACM_DataAvailable             (void);
int32_t  USBD_CDC_ACM_Notify                    (uint16_t stat);
*/

int main( void ) {
  
  /* Initialization of the USDB device. */
  usbd_init();
  while (!usbd_configured()) {
  }
  /* Main loop to echo a character */
  do {
    /* just a little delay to wait for a character instead of USB Nacking. */
    seSysSleepMS( 1 );    
    int32_t ch = USBD_CDC_ACM_GetChar();
    if ( ch != (-1) ) {
      USBD_CDC_ACM_PutChar( ch );
    }
  } while (1);
  
  return seSTATUS_OK;
} 

/**
  * @brief When RTX is not used following functions are called in Interrupt context.
  * Uncomment printfs only for debugging. Alternatively the event functions can set 
  * the global event flags.
  */

void USBD_Reset_Event       (void) { /* printf( "Reset\n"); */ }
void USBD_Suspend_Event     (void) { /* printf( "Suspend\n");*/ }
void USBD_Resume_Event      (void) { /* printf( "Resume\n" );*/ }
void USBD_Configure_Event   (void) { /* printf( "Configure\n"); */}
void USBD_Interface_Event   (void) { /* printf( "Interface\n" );*/ }
void USBD_Feature_Event     (void) {  /* printf( "Feature\n");*/}


/* USB Device CDC ACM class functions called automatically by USBD Core module durinng class initialization */
int32_t  USBD_CDC_ACM_Initialize (void) {
  USBD_CDC_ACM_PortInitialize();
  USBD_CDC_ACM_Reset ();
  return 1; }

/* Functions handling CDC ACM requests (overridden to provide custom
   handling of CDC ACM requests */
int32_t USBD_CDC_ACM_SendEncapsulatedCommand (void){return 1;}
int32_t USBD_CDC_ACM_GetEncapsulatedResponse (void){return 1;}
int32_t USBD_CDC_ACM_SetCommFeature          (uint16_t feat){return 1;}
int32_t USBD_CDC_ACM_GetCommFeature          (uint16_t feat){return 1;}
int32_t USBD_CDC_ACM_ClearCommFeature        (uint16_t feat){return 1;}
int32_t USBD_CDC_ACM_SendBreak               (uint16_t dur){return 1;}







